import axios from 'axios'
import decrypt from '../decrypt'
import { getToken } from 'qtable-auth-hook'

let baseURL = ''
export function setBaseURL(url) {
	baseURL = url
}

const CancelToken = axios.CancelToken
const pendingQueue = []

const removePending = (config, isRequest) => {
	// isRequest标记是请求拦截器还是响应拦截器
	const urlFlag = `${config.url}&${config.method}`
	const index = pendingQueue.findIndex((item) => item.urlFlag === urlFlag)
	if (index === -1) return
	const item = pendingQueue[index]
	if (isRequest && typeof config.noCancel !== 'function') {
		item.canceler() // 对该次接口之前发送的的同名接口执行取消操作
		console.log(config.url, '接口被取消')
	}
	pendingQueue.splice(index, 1) // 把这条记录从数组中移除
}

// create an axios instance
export const service = axios.create({
	timeout: 20000 // request timeout
})

service.interceptors.request.use(
	(config) => {
		config.headers.Authorization = `${getToken()}`
		config.data = config.data || {}
		removePending(config, true)
		config.cancelToken = new CancelToken((canceler) => {
			const isCancelFunction = typeof config.noCancel === 'function'
			if (!config.noCancel || isCancelFunction) {
				// 该请求可被取消
				const obj = {
					urlFlag: `${config.url}&${config.method}`, // url标记，便宜请求结束后从pendingQueue中移除
					canceler // 执行该方法可取消正在pending中的该请求
				}
				if (isCancelFunction) {
					obj.data = config.data // 传递给noCancel回调的参数
				}
				pendingQueue.push(obj)
			}
		})
		// im接口传参为json
		if (config.url.startsWith('/im/')) {
			config.headers['content-type'] = 'application/json; charset=UTF-8'
		} else {
			config.data = Object.entries(config.data)
				.map(([key, value]) => `${key}=${encodeURIComponent(value)}`)
				.join('&')
		}
		return config
	},
	(error) => {
		console.log(error)
	}
)

service.interceptors.response.use(
	({ data: res, config }) => {
		removePending(config, false) // 清除该记录
		if (res.status === 0) {
			// 接口调用成功
			if (res.encrypt_data) {
				// 解密数据
				res.data = decrypt(res.encrypt_data)
			}
			config.successHandler && config.successHandler(res)
			return res
		}
		return Promise.reject(res)
	},
	(error) => {
		if (error.message) console.log(error)
		return Promise.reject(error)
	}
)

// export default service
const maxRetryCount = 2
export default async function request(config) {
	config.baseURL = baseURL
	try {
		if (import.meta.env.MODE === 'development' && config.delay) {
			// 测试接口时间较长的影响
			return new Promise((resolve, reject) => {
				setTimeout(() => {
					service(config).then(resolve).catch(reject)
				}, config.delay)
			})
		}
		return await service(config)
	} catch (err) {
		// 出错重新发送
		if (!err.message) return Promise.reject(err) // 请求被取消时，无message，此时不处理
		if (config.retryCount) {
			if (config.retryCount > maxRetryCount) {
				config.retryCount = maxRetryCount
			}
			return request({
				...config,
				retryCount: config.retryCount - 1
			})
		}
		return Promise.reject(err)
	}
}
export function postRequest(configOrUrl, data) {
	const obj = {
		data,
		method: 'post'
	}
	if (typeof configOrUrl === 'string') {
		obj.url = configOrUrl
	} else if (typeof configOrUrl === 'object') {
		Object.assign(obj, configOrUrl)
	}
	return request(obj)
}

