import { debounce } from '@/tools'

export const popperTransitionDuration = 150
export const mouseleaveCloseDelay = 200 // slot写法下，slotTrigger为hover时有用

export function isAncestor(targetNode, ancestor) { // ancestor: dom 或 class字符串 或 class字符串数组
  const isAncestor = typeof ancestor === 'string'
    ? (targetNode, ancestor) => targetNode.classList && targetNode.classList.contains(ancestor) // class字符串
    : Array.isArray(ancestor)
      ? (targetNode, ancestor) => targetNode.classList && ancestor.some(className => targetNode.classList.contains(className)) // class字符串数组
      : (targetNode, ancestor) => targetNode === ancestor // dom
  while (targetNode) {
    if (isAncestor(targetNode, ancestor)) return targetNode
    targetNode = targetNode.parentNode
  }
  return null
}

export function splitClass(className) {
  const classList = Array.isArray(className) ? className : className.split(' ')
  return classList.filter(Boolean)
}

export const visiblePoppers = [] // 只存储显示状态的popper，没有popper显示时移除全局事件监听

export const scrollHandler = (e) => {
  visiblePoppers.forEach((popper) => {
    if (!popper.isDomPlacement) return
    if (popper.scrollClose && !isAncestor(e.target, popper.$refs.popper)) {
      popper.close() // 当滚动发生在popper内部时不关闭
    } else if (isAncestor(popper.referenceEl, e.target)) { // 引用元素所在容器发生滚动导致reference位置变化
      popper.setPlace(-1)
      const subPopper = visiblePoppers.find(p => isAncestor(p.referenceEl, popper.$refs.popper))
      if (subPopper) {
        subPopper.setPlace(-1)
      }
    }
  })
}
export const resizeHandler = debounce(() => {
  visiblePoppers.forEach((popper) => {
    if (!popper.isDomPlacement) return
    popper.setPlace()
    const subPopper = visiblePoppers.find(p => isAncestor(p.referenceEl, popper.$refs.popper))
    if (subPopper) {
      subPopper.setPlace()
    }
  })
})

export const clickHandler = (e) => {
  // 这里不能使用forEach，for of等循环，使用for循环要倒叙，因为onClickOut中会splice当前索引的popper，导致index混乱，循环漏执行
  for (let i = visiblePoppers.length - 1; i >= 0; i--) {
    visiblePoppers[i].onClickOut(e)
  }
}

export const clickCloseHandler = (e) => {
  // 这里不能使用forEach，for of等循环，使用for循环要倒叙，因为onClickClose中会splice当前索引的popper，导致index混乱，循环漏执行
  for (let i = visiblePoppers.length - 1; i >= 0; i--) {
    visiblePoppers[i].onClickClose(e)
  }
}

export const keydownHandler = (e) => {
  visiblePoppers.forEach((popper) => {
    popper.onKeydown(e)
  })
}

export const closeAllPopper = () => {
  visiblePoppers.forEach(popper => {
    popper.close()
  })
}

// 当页面引入iframe时，点击iframe区域，不会触发当前页面的点击事件，导致popper不能关闭，因此判断当页面有iframe时，创建一个div蒙层
export function createPopperMask() {
  if (!document.querySelector('iframe') || !visiblePoppers.length) return
  let maskEl = document.querySelector('.d-popper-mask')
  if (maskEl) {
    maskEl.style.display = 'block'
  } else { // 还没创建过
    const firstPopperDom = document.querySelector('.d-popper')
    if (!firstPopperDom || !firstPopperDom.parentNode) return
    maskEl = document.createElement('div')
    maskEl.className = 'd-popper-mask'
    maskEl.style.cssText = `position: fixed; left: 0; right: 0; top: 0; bottom: 0; zIndex: ${firstPopperDom.style.zIndex};`
    firstPopperDom.parentNode.insertBefore(maskEl, firstPopperDom)
  }
}

export function hiddenPopperMask() {
  const maskEl = document.querySelector('.d-popper-mask')
  if (maskEl) {
    maskEl.style.display = 'none'
  }
}
